package org.artifactory.queue;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.HaAddon;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.common.property.ArtifactorySystemProperties;
import org.artifactory.opentracing.ArtifactoryTracer;
import org.artifactory.queue.error.PersistentQueueErrorService;
import org.artifactory.queue.execution.QueueTriggeringService;
import org.artifactory.queue.services.PersistentQueueCursorService;
import org.artifactory.queue.services.PersistentQueueCursorServiceImpl;
import org.artifactory.storage.db.event.entity.NodeEventCursorType;
import org.artifactory.storage.db.event.service.NodeEventCursorService;
import org.artifactory.storage.event.EventsService;
import org.jfrog.common.concurrency.ConflictGuard;
import org.jfrog.common.concurrency.ConflictsGuard;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;

import java.util.concurrent.TimeUnit;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyLong;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public abstract class PersistentQueueBaseTest {

    @Mock
    ConflictGuard conflictGuard;
    @Mock
    EventsService eventsService;
    @Mock
    ArtifactoryContext artifactoryContext;
    @Mock
    NodeEventCursorService nodeEventCursorService;
    @Mock
    PersistentQueueCursorService queueCursorService;
    @Mock
    PersistentQueueErrorService queueErrorService;
    @Mock
    AddonsManager addonsManager;

    @BeforeMethod
    public void prepareMocks() throws Exception {
        MockitoAnnotations.openMocks(this);
        when(artifactoryContext.beanForType(EventsService.class)).thenReturn(eventsService);
        when(artifactoryContext.beanForType(PersistentQueueCursorService.class)).thenReturn(queueCursorService);
        when(artifactoryContext.beanForType(PersistentQueueErrorService.class)).thenReturn(queueErrorService);
        when(artifactoryContext.beanForType(ArtifactoryTracer.class)).thenReturn(mock(ArtifactoryTracer.class));
        when(artifactoryContext.beanForType(QueueTriggeringService.class))
                .thenReturn(mock(QueueTriggeringService.class));
        ArtifactoryContextThreadBinder.bind(artifactoryContext);
        final ArtifactoryHome artifactoryHome = mock(ArtifactoryHome.class);
        final ArtifactorySystemProperties mockProperties = new ArtifactorySystemProperties();
        when(artifactoryHome.getArtifactoryProperties()).thenReturn(mockProperties);
        ArtifactoryHome.bind(artifactoryHome);
        HaAddon haAddon = mock(HaAddon.class);
        ConflictsGuard conflictsGuard = mock(ConflictsGuard.class);
        when(conflictsGuard.tryToLock(anyString(), anyLong(), any(TimeUnit.class))).thenReturn(true);
        when(haAddon.getConflictsGuard(eq(PersistentQueueCursorServiceImpl.CREATE_CURSOR_CONFLICTS_GUARD)))
                .thenReturn(conflictsGuard);
        when(addonsManager.addonByType(eq(HaAddon.class))).thenReturn(haAddon);
    }

    @AfterMethod
    public void cleanup() {
        ArtifactoryHome.unbind();
        ArtifactoryContextThreadBinder.unbind();
    }

    protected QueueConfiguration createIncrementalIndexQueue() {
        return getIncrementalIndexBuilder().build();
    }

    protected QueueConfiguration createIncrementalIndexQueue(QueueEventImplementor implementor, int maxRetries) {
        return getIncrementalIndexBuilder()
                .implementor(implementor)
                .maxErrorRetires(maxRetries)
                .build();
    }

    protected QueueConfiguration.QueueConfigurationBuilder getIncrementalIndexBuilder() {
        return QueueConfiguration.builder()
                .implementor((events, key) -> {})
                .cursorType(NodeEventCursorType.INCREMENTAL_INDEX);
    }
}
