package org.artifactory.repo;

import com.fasterxml.jackson.annotation.JsonFilter;
import com.google.common.collect.Lists;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.artifactory.descriptor.repo.FederatedMember;
import org.artifactory.descriptor.repo.FederatedRepoDescriptor;

import java.util.List;
import java.util.stream.Collectors;

/**
 * Federated repository configuration
 *
 * @author Gidi Shabat
 * @author Andrei Komarov
 * @see FederatedRepoDescriptor
 */
@Data
@EqualsAndHashCode(callSuper = false)
@JsonFilter("typeSpecificFilter")
@NoArgsConstructor
public class FederatedRepositoryConfigurationImpl extends LocalRepositoryConfigurationImpl
        implements FederatedRepositoryConfiguration {

    public static final String TYPE = "federated";
    private List<FederatedMemberConfiguration> members;

    public FederatedRepositoryConfigurationImpl(FederatedRepoDescriptor federatedRepoDescriptor) {
        super(federatedRepoDescriptor, TYPE);
        List<FederatedMember> descriptorMembers = federatedRepoDescriptor.getFederatedMembers();
        if (descriptorMembers != null && !descriptorMembers.isEmpty()) {
            setMembers(descriptorMembers.stream().map(this::toFederatedMemberConfig).collect(Collectors.toList()));
        } else {
            setPropertySets(Lists.newArrayList());
        }
    }

    private FederatedMemberConfiguration toFederatedMemberConfig(FederatedMember a) {
        return new FederatedMemberConfiguration(a.getUrl(), a.getMode());
    }
}
